#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Johan Hake (johan.hake@gmail.com)"
__copyright__ = "Copyright (C) 2008 Johan Hake"
__license__  = "GNU LGPL Version 3.0 or any later"
__date__ = "2008-11-23 -- 2009-08-14"

import os, glob

from diffsim import load, save
import re
import numpy
import pylab
from subprocess import Popen

def analyze_res(res,hight_plot=False):
    data_files = [os.path.basename(f) for f in glob.glob("data/collected_stochastic_voltage_clamp_h*_res_%d*"%res)]
    data_files.sort()
    if filter(lambda s:"h100" in s,data_files):
        print "Flipping data for h = 100"
        h100 = data_files.pop(1)
        data_files.append(h100)
        print "first",data_files[0]
        print "last",data_files[-1]
    
    medians = numpy.zeros(len(data_files))
    means   = numpy.zeros(len(data_files))
    over_50 = numpy.zeros(len(data_files))
    hights  = numpy.zeros(len(data_files))

    print data_files
    
    tstops = {}
    for i, f in enumerate(data_files):
        if hight_plot:
            pylab.figure()
        h = float(re.findall(r"_h([0-9.]+)",f)[0])
        hights[i] = h
        print "h:",h, "res:",res
        data = load(f)['diffsim_data']
        tstop = numpy.zeros(len(data['time']))
        for j, run in enumerate(data['time']):
            tstop[j] = run['tstop']
        tstops[h] = {}
        tstops[h]['tstop'] = tstop
        #print "Mean:",numpy.mean(tstop),means, i
        means[i]   = numpy.mean(tstop)
        medians[i] = numpy.median(tstop)
        over_50[i] = (tstop==50.0).sum()
        hist = numpy.histogram(tstop,10,(0.,50.),True,new=True)
        tstops[h]['hist'] = hist
        
        if hight_plot:
            # barplot
            pylab.bar(hist[1],hist[0],5.0)
            pylab.title('Cleft hight: %s, Resolution: %d'%(str(h),res))
            pylab.xlabel('Release time [ms]')
            info_str = """Mean: %s
    Median: %s
    Num 50ms: %d"""%(means[i],medians[i],over_50[i])
            pylab.text(0.3, 0.9,info_str,
                       horizontalalignment='center',
                       verticalalignment='center',
                       transform = pylab.gca().transAxes,
                       )
        
            plot_basename = 'figures/release_time_h%s_res%d'%(str(h),res)
            pylab.savefig(plot_basename + ".eps")
            Popen("eps2pdf  %s.eps --outfile  %s.pdf"%\
                  (plot_basename,plot_basename),shell=True)
    
    pylab.figure()
    pylab.plot(hights,means,'d', hights, medians,'s')
    #hights = [h for h in hights if h not in [12.5, 17.5, 25., 35., 70., 90.]]
    #width = 3
    #pylab.boxplot([tstops[h]['tstop'] for h in hights],positions=hights, widths = width)
    #pylab.gca().set_xlim(hights[0]-width*2,hights[-1]+width*2)
    pylab.title('Mean and median release times for resolution %d'%res)
    pylab.xlabel('hights [nm]')
    pylab.ylabel('release time [ms]')
    pylab.legend(("mean","median"))
    
    plot_basename = 'figures/mean_release_times_res%d'%res
    pylab.savefig(plot_basename + ".eps")
    Popen("eps2pdf  %s.eps --outfile  %s.pdf"%\
          (plot_basename,plot_basename),shell=True)

def collect_res_data(res):
    data_files = [os.path.basename(f) for f in glob.glob("data/stochastic_voltage_clamp_h*_res_%d*"%res)]
    assert(data_files)
    data_files.sort()
    data_collect = {}
    for f in data_files:
        h = float(re.findall(r"_h([0-9.]+)",f)[0])
        data = load(f)['diffsim_data']
        if not data_collect.has_key(h):
            data_collect[h] = data
        else:
            data_collect[h]['time'].extend(data['time'])
            #data_collect[h]['StochasticTimeStepper'].append(data['StochasticTimeStepper'])
    
    for h, data in data_collect.iteritems():
        print "Data length %s: %d"%(str(h),len(data['time']))
        save("collected_stochastic_voltage_clamp_h%s_r100_res_%d_vec_deg_1_SUPG_TYPE_3_tau_0.60"%(str(h),res), diffsim_data = data)
    


if __name__ == "__main__":
    #collect_res_data(0)
    #collect_res_data(2)
    analyze_res(0)
    #analyze_res(2)
    
