#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Johan Hake (johan.hake@gmail.com)"
__copyright__ = "Copyright (C) 2008 Johan Hake"
__license__  = "GNU LGPL Version 3.0 or any later"
__date__ = "2008-02-19 -- 2008-09-12"

import sys, os, time

TIME_FORMAT = "%Y.%m.%d-%H.%M.%S"

def present_time_str():
    return time.strftime(TIME_FORMAT)

def setup_log_file(res,h,radius):
    filename = os.path.join("log",
                "stochastic_voltage_clamp_res%s_h%s_radius%s_%s.log"%
                            (str(res),str(h),str(radius),present_time_str()))
    print filename
    
    log_file = open(filename,'w')
    sys.stdout = log_file
    sys.stderr = log_file

def restore_stdout():
    assert(sys.__stdout__ != sys.stdout)
    #if not sys.stdout.closed:
    #    sys.stdout.close()
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__
    
if __name__ == "__main__":
    if len(sys.argv) != 4:
        raise RuntimeError, "Expected 3 optional arguements"

    res, h, num_runs = tuple(sys.argv[1:])

    res      = eval(res)
    h        = eval(h)
    radius   = 100
    num_runs = eval(num_runs)
    
    setup_log_file(res,h,radius)
    
    from stochastic_voltage_clamp import *
    from diffsim.logger import *
    from diffsim.disk   import present_time_str
    set_log_level(INFO,'all')
    
    params = default_params()
    params.lcc_channels        = [2,3,4]
    params.lcc_amp             = 0.2
    params.constant_flux       = 1
    params.ryr_channels        = range(20,36)
    params.tstop               = 50.0
    params.screen_logger       = 0
    params.num_runs            = num_runs
    
    # Script related params
    params.hight               = h
    params.resolution          = res
    params.radius              = radius
    params.SUPG_type           = 3
    params.tau_dict            = {0:0.6,1:0.6,2:0.6}#{0:0.22,1:0.28,2:0.38}, # From experiments
    params.voltage_step        = 10.0
    params.holding_pot         = -80.0
    params.clamp_time          = 0.1
    
    if res == 0:
        params.markov_model_scale = 1.0
        params.phi_0              = 0.0
    else:
        params.markov_model_scale = 1.5
        params.phi_0              = -2.2
    
        
    objs, case_name = setup(params)
    
    run_system(params)

    restore_stdout()
