#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Johan Hake (johan.hake@gmail.com)"
__copyright__ = "Copyright (C) 2008 Johan Hake"
__license__  = "GNU LGPL Version 3.0 or any later"
__date__ = "2008-02-19 -- 2008-09-12"

# NOTE: We need to import scipy.optimize before dolfin!!
#       Otherwise -> segfault
import scipy.optimize


from diffsim import *
from diffsim.dyadiccleft import *

from logger import *
from os import path

set_log_level(DEBUG,ALL)
set_log_level(CRITICAL,"instant")
#et_log_level(INFO,'diffsim.diffusiveligand')

# Define some constant rate functions
def rate_func_0(p,var):
    return 2.0

def rate_func_1(p,var):
    return 4.0

def rate_func_2(p,var):
    return 1.0

def rate_func_3(p,var):
    return 6.0

class MarkovChainTester1(DiscreteMarkovChain):
    _params = ParameterDict()
    _rate_func_map = {(0,1):rate_func_0,(1,0):rate_func_1}
    _ext_vars = {}
    _state_values = {1:1}

class MarkovChainTester2(DiscreteMarkovChain):
    _params = ParameterDict()
    _rate_func_map = {(0,1):rate_func_2,(1,0):rate_func_3}
    _ext_vars = {}
    _state_values = {1:1}

class MarkovModelTester(DiscreteMarkovModel):
    _DiscreteMarkovChains = [MarkovChainTester1,MarkovChainTester2]
    _dmc_combine = mul
    
# Ca specific parameters
c0 = 0.1
D_Ca = 1e5

# Time dependent parameters
tstop     = 10.0
dt_min    = 0.000001
use_cut_plane_plot  = 0
use_plot_TT         = 0
use_plot_SR         = 0
plot_range          = [0,2000]#[0,5]
channels            = [2,3]
cytosole            = 1
run_times           = 1

case_name = path.basename(__file__).split('.')[0]
mesh_name = "mesh/cleft_mesh_3D_r20_res_1.xml"

dd = DiffusionDomain("Dyadic_cleft",mesh_name,vector_degree = 2)
dl = DiffusiveLigand(dd.name(),"Ca",c0)
sf = StaticField("Bi_lipid_field",dd.name())

times  = [0,tstop/2]
values = [[0,1],[1,0]]
for i, m in enumerate(channels):
    ConstantChannelFlux("LCC",dd.name(), "Ca",m, amp = 0.2,
                        activator = MarkovModelTester)

if cytosole:
    ConcDepLinearFluxBoundary("Cytosole",dd.name(),"Ca",100, u0 = c0,dn = 5.0)

if use_cut_plane_plot:
    p = Plotter(dl.name(),plot_range = plot_range,
                cutplane_origin = (0,0,0),
                cutplane_normal = (0,1,0),
#                save_png = True,
                )

if use_plot_SR:
    p = Plotter(dl.name(),plot_range = plot_range,
                elevate = -60)

if use_plot_TT:
    p = Plotter(dl.name(),plot_range = plot_range,
                elevate = -60,azimuth = 180)

sl = ValueLogger(dl.name(),[1,2,3,20,100],[MIN,MAX,MEAN,STD],to_file=False)

# Setting the parameters
params = get_params()

params.time.dt_min          = dt_min
params.time.dt_start        = dt_min
params.time.tstop           = tstop
params.solver.tol_rel       = 0.001
params.solver.tol_abs       = 0.0001
params.solver.tol_ss        = 0.001
params[sf.name()].phi_0     = -2.2
params[sf.name()].SUPG_type = 3
params[sf.name()].tau       = 0.20

init_system()
if run_times:
    run_sim(run_times,case_name)

#o = DiffsimObject.objects()

