class FieldDirection: public dolfin::Function
{
public:
  dolfin::Function* field;
  double      D;
  FieldDirection(dolfin::Mesh & mesh):
    dolfin::Function(mesh)
  {
    field = 0;
    D     = 1.0;
  }

  void eval(double* v, const double* x) const
  {
    if (!field)
       dolfin::error("Attach a field function.");
    double field_norm = 0.0;
    double tau = 0.0;
    double h = cell().diameter();
    field->eval(v,x);
    for (dolfin::uint i = 0;i < field->dim(0); ++i)
      field_norm += v[i]*v[i];
    field_norm = sqrt(field_norm);
    double PE = 0.5*field_norm * h/D;
    if (fabs(PE) > DOLFIN_EPS)
      tau = 1/tanh(PE)-1/PE;
    for (dolfin::uint i = 0;i < field->dim(0); ++i)
      v[i] *= 0.5*h*tau/field_norm;
  }
  
  dolfin::uint rank() const
  {
    return 1;
  }
  
  dolfin::uint dim(dolfin::uint i) const
  {
    switch(i) {
      case 0: return 3;
    }
    throw std::runtime_error("Invalid dimension i in dim(i).");
  }
};
