// Copyright (C) 2008 Johan Hake.
// Licensed under the GNU LGPL Version 2.1.

/* Streamline Upwind Petrow/Galerkin stabilizing coefficient function

  The perturbed testfunction, v', is given by

     v' = v + s*grad(v) 

  where the stabilizing term s is given by
 
    s = tau *h*tau_l/(2*|a|)*a

  where a is the field, tau a global tuning parameter, (not inlcuded here),
   and tau_l the local stabilization parameter

 */
class Stab: public Expression {
public:
  boost::shared_ptr<Expression> field;
  double D; 
  Stab():
    Expression(3), D(1.0){}
  
  void eval(double* v, const dolfin::Data& data) const
  {
    if (!field)
      error("Attach a field function.");
    double field_norm = 0.0;
    double tau = 0.0;
    double h = data.cell().diameter();
    field->eval(v, data.x);
    for (uint i = 0;i < data.x.size(); ++i)
      field_norm += v[i]*v[i];
    field_norm = sqrt(field_norm);
    double PE = 0.5*field_norm * h/D;
    if (PE > DOLFIN_EPS)
      tau = 1/tanh(PE)-1/PE;
    for (uint i = 0;i < data.x.size(); ++i)
      v[i] *= 0.5*h*tau/field_norm;
  }
};
