class StabH: public dolfin::Function
{
public:
  dolfin::Function* field;
  double D;
  StabH(dolfin::Mesh & mesh):
    dolfin::Function(mesh)
  {
    field = 0;
    D     = 1.0;
  }

  void eval(double* v, const double* x) const
  {
    if (!field)
       dolfin::error("Attach a field function.");
    double field_norm = 0.0;
    double tau = 0.0;
    double h = cell().diameter();
    double val[3];
    field->eval(val,x);
    for (dolfin::uint i = 0;i < field->dim(0); ++i)
      field_norm += val[i]*val[i];
    field_norm = sqrt(field_norm);
    
    double PE = 0.5*field_norm * h/D;
    if (fabs(PE) > DOLFIN_EPS)
      tau = 1/tanh(PE)-1/PE;
    //if (tau>.5)
    //  printf("Tau: %0.2f, z: %0.2f\n",tau,x[2]);
    v[0] = 0.5*tau*h/field_norm;
  }
};
