#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Johan Hake (johan.hake@gmail.com)"
__copyright__ = "Copyright (C) 2008 Johan Hake"
__license__  = "GNU LGPL Version 3.0 or any later"
__date__ = "2008-06-05 -- 2008-11-18"

"""A minimal time handler for providing timing of multiple processes"""

__all__ = ["tic","toc","report","registered_processes","clear_report"]

import time
import numpy

_start_times = {}
_start_times["module import"] = time.time()

_intermediate_times = {}
_report = {}

def tic(name = None):
    """ Starts a timing of a process

    @param name : The name of the proces
    """
    name = name or ""
    now = time.time()
    _start_times[name] = now
    _intermediate_times[name] = now
    if not _report.has_key(name):
        _report[name] = []

def toc(name = None, from_tic = False):
    """ Return the time from the last "toc" of a process

    @param name     : The name of the process
    @param from_tic : If True the time from the process' tic is returned.
    """
    name = name or ""
    if name in _start_times.keys():
        now = time.time()
        if from_tic:
            diff = now - _start_times[name]
        else:
            diff = now - _intermediate_times[name]
        _intermediate_times[name] = now
        _report[name] += [diff]
        return diff
    else:
        raise RuntimeError, "No process started with name %s"%name

def report(name = None):
    """ Return a numpy array of reported of timings

    @param name  : The name of the process
    """
    name = name or ""
    if not _report.has_key(name):
        raise RuntimeError, "No process with name %s registered."%name
    return numpy.array(_report[name])

def clear_report(name = None):
    name = name or ""
    if not _report.has_key(name):
        raise RuntimeError, "No process with name %s registered."%name
    _report[name] = []

def registered_processes():
    " Return the registered processes."
    return _report.keys()


if __name__ == "__main__":
    print "Start timing"
    tic()
    time.sleep(0.2)
    print "Start process A"
    tic("A")
    time.sleep(0.2)
    print "Time since start",toc()
    time.sleep(0.2)
    print "Time since last toc()",toc()
    time.sleep(0.2)
    print "Process A since start:",toc("A")
    time.sleep(0.2)
    print "Time since start",toc(from_tic=True)
    print "Process A since start:",toc("A",from_tic=True)

    
